# 游密科技 WebRTC SDK for HTML5

本 SDK 对 WebRTC 和游密信令服务器、游密 turn 服务器进行统一封装，开发者申请了游密 appKey 后，使用本 SDK 即可简单快速地实现实时视频和实时语音功能。

**兼容说明**

操作系统平台 |浏览器/webview|版本要求|接收（播放）|发送（上麦）|屏幕共享|备注
-------|------|------|------|-------|------|------
macOS|桌面版 Safari 浏览器|11+|支持|支持|支持（需要 Safari13+ 版本）| - |
macOS|桌面版 Chrome 浏览器|56+|支持|支持|支持（需要 Chrome72+ 版本）| - |
macOS|桌面版 Firefox 浏览器|56+|支持|支持|支持（需要 Firefox66+ 版本）| - |
macOS|桌面版 Edge 浏览器|80+|支持|支持|支持| - |
Windows|桌面版 Chrome 浏览器|56+|支持|支持|支持（需要 Chrome72+ 版本）| - |
Windows|桌面版 QQ 浏览器（极速内核）|10.4+|支持|支持|不支持| - |
Windows|桌面版 Firefox 浏览器|56+|支持|支持|支持（需要 Firefox66+ 版本）| - |
Windows|桌面版 Edge 浏览器|80+|支持|支持|支持| - |
iOS 11.1.2+|移动版 Safari 浏览器|11+|支持|支持|不支持| - |
iOS 12.1.4+|微信内嵌网页| - |支持|不支持|不支持| - |
iOS 14.3+|微信内嵌网页|6.5+（微信版本）|支持|支持|不支持| - |
Android|移动版 QQ 浏览器| - |不支持|不支持|不支持| - |
Android|移动版 UC 浏览器| - |不支持|不支持|不支持| - |
Android|微信内嵌网页（TBS 内核）| - |支持|支持|不支持| - |
Android|微信内嵌网页（XWEB 内核）| - |支持|支持|不支持| - |


**注意**

- 由于 H.264 版权限制，华为系统的 Chrome 浏览器和以 Chrome WebView 为内核的浏览器均不支持, 桌面浏览器端 SDK 的正常运行。

## 用法

使用本 SDK 只要四步。

### 第一步：引入 js 文件

** 直接引入 `ymrtc.umd.min.js`：

```html
<script src="path/to/ymrtc.umd.min.js"></script>
```

### 第二步：初始化

本 SDK 存在一个类：`YMRTC`，只要通过`YMRTC.createClient()`实例化一个`RTCClient`类，就可以使用本 SDK 的全部功能。

若使用**方法一**直接引入 js 文件，可以直接访问 `YMRTC` 类：

```javascript
var rtc = YMRTC.createClient({
    mode: 'rtc',
    codec: 'h264,vp8',
    debug: false,  // 开启debug日志, 可选
    dev: false     // 使用测试环境, 可选
});
```

本 SDK 同时支持 CommonJS 或 AMD 标准，除了上述直接引用，也可以使用这些标准方法初始化：

```javascript
// CommonJS 规范
var YMRTC = require('path/to/ymrtc.cjs.min.js');
var rtc = YMRTC.createClient({
    mode: 'rtc',
    codec: 'h264,vp8',
    debug: false,  // 开启debug日志, 可选
    dev: false     // 使用测试环境, 可选
});
```

或者

```javascript
// AMD 规范
require(['path/to/ymrtc.umd.min.js'], function(YMRTC) {
    var rtc = YMRTC.createClient({
        mode: 'rtc',
        codec: 'h264,vp8',
        debug: false,  // 开启debug日志, 可选
        dev: false     // 使用测试环境, 可选
    });
});
```

或者，若使用 ES6 或 TypeScript，可以使用 `import`：

```javascript
import YMRTC from 'path/to/ymrtc.es.min';
const rtc = YMRTC.createClient({
    mode: 'rtc',
    codec: 'h264,vp8',
    debug: false,  // 开启debug日志, 可选
    dev: false     // 使用测试环境, 可选
});
```

或者，若使用**方法二**使用 npm 引入，则使用本 SDK 的 npm 名称：

```typescript
import YMRTC from 'youme-webrtc-h5';
const rtc = YMRTC.createClient({
    mode: 'rtc',
    codec: 'h264,vp8',
    debug: false,  // 开启debug日志, 可选
    dev: false     // 使用测试环境, 可选
});
```

### 第三步：初始化、用户登录、加入房间、初始化本地媒体（本地摄像头和麦克风）

```javascript
// 初始化
rtc.init(APPKEY, () => {
  /** success */
}, (err) => {
  /** error */
})

// 加入房间
rtc.join('token', '[房间号，支持数字、字母、下划线组合]', '[用户id，支持数字、字母、下划线组合]', '', () => {
    /** success */
}, () => {
    /** error */
});

// 初始化本地媒体，然后把本地媒体放入
```

更多控制 API 请参看下文的 API 文档，亦可以参看示例代码。

### 第四步：监听事件，播放其他用户传来的视频和语音

# API 文档

以下是完整的 API 文档。
## 类 (class)

<dl>
<dt><a href="#YMRTC">YMRTC</a></dt>
<dd><p>游密Webrtc SDK</p></dd>
<dt><a href="#RTCClient">RTCClient</a></dt>
<dd></dd>
<dt><a href="#Stream">Stream</a></dt>
<dd><p>流对象</p></dd>
</dl>

## 自定义类型 (typedef)

<dl>
<dt><a href="#ClientConfig">ClientConfig</a> : <code>Object</code></dt>
<dd><p>定义 createClient 中的 config 参数的接口。</p></dd>
<dt><a href="#StreamSpec">StreamSpec</a> : <code>Object</code></dt>
<dd><p>创建流的配置参数</p></dd>
<dt><a href="#GetDevicesSuccCb">GetDevicesSuccCb</a> ⇒ <code>void</code></dt>
<dd><p>getDevices 成功回调</p></dd>
<dt><a href="#GetDevicesFailCb">GetDevicesFailCb</a> ⇒ <code>void</code></dt>
<dd><p>getDevices 失败回调</p></dd>
<dt><a href="#LiveTranscoding">LiveTranscoding</a> : <code>Object</code></dt>
<dd><p>管理 CDN 直播推流转码的接口
调用 <code>setLiveTranscoding</code> 时可以通过该接口设置直播推流。</p></dd>
<dt><a href="#NetworkType">NetworkType</a> : <code>Object</code></dt>
<dd><p>网络类型</p>
<ul>
<li>&quot;bluetooth&quot;：蓝牙网络</li>
<li>&quot;cellular&quot;：蜂窝移动数据网络</li>
<li>&quot;ethernet&quot;：以太网</li>
<li>&quot;none&quot;：没有网络</li>
<li>&quot;wifi&quot;：Wi-Fi</li>
<li>&quot;wimax&quot;：WiMax</li>
<li>&quot;other&quot;：其他网络类型</li>
<li>&quot;unknown&quot;：未知网络类型</li>
<li>&quot;UNSUPPORTED&quot;：浏览器不支持获取网络类型</li>
</ul>
<p>该功能仅支持 Chrome 61 及以上版本，且无法保证兼容性</p></dd>
<dt><a href="#SubscribeOptions">SubscribeOptions</a> : <code>Object</code></dt>
<dd><p>subscribe方法需要的options参数</p></dd>
<dt><a href="#getCameras">getCameras</a> ⇒ <code>void</code></dt>
<dd><p>getCameras方法回调函数类型</p></dd>
</dl>

<a name="YMRTC"></a>

## YMRTC
<p>游密Webrtc SDK</p>

**类型**: 全局类(class)  

* [YMRTC](#YMRTC)
    * [.createClient(config)](#YMRTC.createClient) ⇒ [<code>RTCClient</code>](#RTCClient)
    * [.checkSystemRequirements()](#YMRTC.checkSystemRequirements) ⇒ <code>boolean</code>
    * [.getDevices(callback, [callback_Error])](#YMRTC.getDevices) ⇒ <code>void</code>
    * [.getAudioDevices(callback, [callback_Error])](#YMRTC.getAudioDevices) ⇒ <code>void</code>
    * [.createHark(stream, HarkOption)](#YMRTC.createHark) ⇒ <code>Hark</code>

<a name="YMRTC.createClient"></a>

### YMRTC.createClient(config) ⇒ [<code>RTCClient</code>](#RTCClient)
<p>该方法用于创建客户端，在每次会话里仅调用一次。</p>

**类型**: 静态方法(function)，来自 [<code>YMRTC</code>](#YMRTC)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| config | [<code>ClientConfig</code>](#ClientConfig) | <p>定义客户端的属性</p> |

**示例**  
```js
YMRTC.createClient(config)
```
<a name="YMRTC.checkSystemRequirements"></a>

### YMRTC.checkSystemRequirements() ⇒ <code>boolean</code>
<p>该方法检查 Web SDK 对正在使用的浏览器的适配情况。
你需要在创建音视频对象 (createClient)之前调用该方法，用以检查 Web SDK 对正在使用的浏览器的适配情况
浏览器类型和版本均会影响返回结果,部分 Chromium 内核的浏览器（如 QQ 浏览器等），我们未做全量测试，如需使用，可以进行尝试。</p>

**类型**: 静态方法(function)，来自 [<code>YMRTC</code>](#YMRTC)  
**返回值**: <code>boolean</code> - <ul>
<li>是否支持</li>
</ul>  
**示例**  
```js
const support = YMRTC.checkSystemRequirements()
```
<a name="YMRTC.getDevices"></a>

### YMRTC.getDevices(callback, [callback_Error]) ⇒ <code>void</code>
<p>该方法枚举可用的媒体输入/输出设备，比如麦克风、摄像头、耳机等。
调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的媒体设备以及设备信息。</p>

**类型**: 静态方法(function)，来自 [<code>YMRTC</code>](#YMRTC)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| callback | [<code>GetDevicesSuccCb</code>](#GetDevicesSuccCb) | <p>成功回调</p> |
| [callback_Error] | [<code>GetDevicesFailCb</code>](#GetDevicesFailCb) | <p>失败回调</p> |

<a name="YMRTC.getAudioDevices"></a>

### YMRTC.getAudioDevices(callback, [callback_Error]) ⇒ <code>void</code>
<p>该方法枚举可用的媒体输入/输出音频设备，比如麦克风、耳机等。
调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的媒体设备以及设备信息。</p>

**类型**: 静态方法(function)，来自 [<code>YMRTC</code>](#YMRTC)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| callback | [<code>GetDevicesSuccCb</code>](#GetDevicesSuccCb) | <p>成功回调</p> |
| [callback_Error] | [<code>GetDevicesFailCb</code>](#GetDevicesFailCb) | <p>失败回调</p> |

<a name="YMRTC.createHark"></a>

### YMRTC.createHark(stream, HarkOption) ⇒ <code>Hark</code>
<p>该方法创建音频事件监听
调用成功后返回Hark对象，可监听：
.on(&quot;speaking&quot;,()=&gt;{开始讲话})
.on(&quot;stopped_speaking&quot;,()=&gt;{停止讲话})
.on(&quot;volume_change&quot;,(volume,threshold)=&gt;{音量变化}) , 音量范围范围：0.0到1.0
stop() 结束监听</p>

**类型**: 静态方法(function)，来自 [<code>YMRTC</code>](#YMRTC)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| stream | [<code>Stream</code>](#Stream) | <p>绑定的流</p> |
| HarkOption | <code>options</code> | <p>只为监听讲话事件和音量，传{}即可</p> |

<a name="RTCClient"></a>

## RTCClient
**类型**: 全局类(class)  

* [RTCClient](#RTCClient)
    * [new RTCClient(config)](#new_RTCClient_new)
    * [.init(appkey, appid, [onSuccess], [onFaialure])](#RTCClient+init) ⇒ <code>void</code>
    * [.join(token, channel, uid, timestamp, [optionalInfo], [onSuccess], [onFailure])](#RTCClient+join) ⇒ <code>void</code>
    * [.kickout(kicked_user, is_admin, kick_time, [onSuccess], [onFailure])](#RTCClient+kickout) ⇒ <code>void</code>
    * [.leave([onSuccess], [onFailure])](#RTCClient+leave) ⇒ <code>void</code>
    * [.createStream(spec)](#RTCClient+createStream) ⇒ [<code>Stream</code>](#Stream)
    * [.publish(stream, [onSuccess], onFailure)](#RTCClient+publish) ⇒ <code>void</code>
    * [.unpublish(stream, [onSuccess], [onFailure])](#RTCClient+unpublish) ⇒ <code>void</code>
    * [.subscribe(stream, [options], [onSuccess], [onFailure])](#RTCClient+subscribe)
    * [.unsubscribe(stream, [onSuccess], [onFailure])](#RTCClient+unsubscribe) ⇒ <code>void</code>
    * [.setClientRole(role, [callback])](#RTCClient+setClientRole) ⇒ <code>void</code>
    * [.sendMessage()](#RTCClient+sendMessage)
    * [.startLiveStreaming(url)](#RTCClient+startLiveStreaming) ⇒ <code>void</code>
    * [.setLiveTranscoding(coding)](#RTCClient+setLiveTranscoding) ⇒ <code>void</code>
    * [.stopLiveStreaming(url)](#RTCClient+stopLiveStreaming) ⇒ <code>void</code>
    * ~~[.getNetworkStats(callback)](#RTCClient+getNetworkStats)~~
    * [.getRecordingDevices(callback)](#RTCClient+getRecordingDevices) ⇒ <code>void</code>
    * [.getPlayoutDevices(callback)](#RTCClient+getPlayoutDevices) ⇒ <code>void</code>
    * [.getCameras(callback)](#RTCClient+getCameras) ⇒ <code>void</code>
    * [.setRePubAndSub(flag)](#RTCClient+setRePubAndSub) ⇒ <code>void</code>
    * ["event:stream-added"](#RTCClient+event_stream-added)
    * ["event:stream-removed"](#RTCClient+event_stream-removed)
    * ["event:peer-online"](#RTCClient+event_peer-online)
    * ["event:peer-leave"](#RTCClient+event_peer-leave)
    * ["event:request-unmute-audio"](#RTCClient+event_request-unmute-audio)
    * ["event:request-mute-audio"](#RTCClient+event_request-mute-audio)
    * ["event:unmute-audio"](#RTCClient+event_unmute-audio)
    * ["event:mute-audio"](#RTCClient+event_mute-audio)
    * ["event:unmute-video"](#RTCClient+event_unmute-video)
    * ["event:mute-video"](#RTCClient+event_mute-video)
    * ["event:client-role-changed"](#RTCClient+event_client-role-changed)
    * ["event:stream-published"](#RTCClient+event_stream-published)
    * ["event:stream-unpublished"](#RTCClient+event_stream-unpublished)
    * ["event:stream-subscribed"](#RTCClient+event_stream-subscribed)
    * ["event:stream-updated"](#RTCClient+event_stream-updated)
    * ["event:media-stats-update"](#RTCClient+event_media-stats-update)
    * ["event:local-media-stats-update"](#RTCClient+event_local-media-stats-update)
    * ["event:connection-state-change"](#RTCClient+event_connection-state-change)
    * ["event:reconnect"](#RTCClient+event_reconnect)
    * ["event:connected"](#RTCClient+event_connected)
    * ["event:error"](#RTCClient+event_error)
    * ["event:kicked-other-user"](#RTCClient+event_kicked-other-user)
    * ["event:kicked-user"](#RTCClient+event_kicked-user)
    * ["event:kickout-failed"](#RTCClient+event_kickout-failed)
    * ["event:joined"](#RTCClient+event_joined)
    * ["event:joinroom_failed"](#RTCClient+event_joinroom_failed)

<a name="new_RTCClient_new"></a>

### new RTCClient(config)
<p>私有</p>


| 参数 | 类型 | 描述 |
| --- | --- | --- |
| config | [<code>ClientConfig</code>](#ClientConfig) | <p>构造参数</p> |

<a name="RTCClient+init"></a>

### rtcClient.init(appkey, appid, [onSuccess], [onFaialure]) ⇒ <code>void</code>
<p>初始化客户端对象
使用该方法初始化客户端对象。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| appkey | <code>string</code> | <p>项目appkey, 在https://youme.im 官网可查</p> |
| appid | <code>number</code> | <p>项目appid, 在https://youme.im 官网可查</p> |
| [onSuccess] | <code>function</code> | <p>可选，成功回调, () =&gt; void;</p> |
| [onFaialure] | <code>function</code> | <p>可选，失败回调, (err: string) =&gt; void</p> |

**示例**  
```js
client.init('appkey', () => {}, (err) => {})
```
<a name="RTCClient+join"></a>

### rtcClient.join(token, channel, uid, timestamp, [optionalInfo], [onSuccess], [onFailure]) ⇒ <code>void</code>
<p>加入频道
调用该方法加入频道时，本地会触发 client.on(&quot;connected&quot;) 和 client.on(&quot;connection-state-change&quot;) 回调；
通信场景下的用户和直播场景下的主播加入频道后，远端会触发 client.on(&quot;peer-online&quot;) 回调；
tokenV3计算方式：token=sha1(apikey+appkey+roomid+userid+timestamp)</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 默认值 | 描述 |
| --- | --- | --- | --- |
| token | <code>string</code> |  | <p>token，非tokenV3输入空串</p> |
| channel | <code>string</code> |  | <p>频道</p> |
| uid | <code>string</code> \| <code>null</code> |  | <p>用户id，不填则sdk创建随机的uid</p> |
| timestamp | <code>number</code> |  | <p>tokenV3时间戳，非tokenV3输入0</p> |
| [optionalInfo] | <code>string</code> | <code>&quot;&#x27;&#x27;&quot;</code> | <p>可选信息</p> |
| [onSuccess] | <code>function</code> |  | <p>可选；成功回调, (uid: number | string) =&gt; void</p> |
| [onFailure] | <code>function</code> |  | <p>可选；失败回调, (err: string) =&gt; void</p> |

**示例**  
```js
client.join('token', 'channel', 'uid', '', (uid) => {}, (err) => {})
```
<a name="RTCClient+kickout"></a>

### rtcClient.kickout(kicked_user, is_admin, kick_time, [onSuccess], [onFailure]) ⇒ <code>void</code>
<p>踢人
调用该方法踢人时，本端和远端都会触发 client.on(&quot;kicked-other-user&quot;)回调；
被踢的人会触发 client.on(&quot;kicked-user&quot;) 回调</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| kicked_user | <code>string</code> | <p>被踢的用户</p> |
| is_admin | <code>string</code> | <p>是否是管理员踢人</p> |
| kick_time | <code>string</code> | <p>被踢的时长</p> |
| [onSuccess] | <code>function</code> | <p>可选；成功回调, (uid: number | string) =&gt; void</p> |
| [onFailure] | <code>function</code> | <p>可选；失败回调, (err: string) =&gt; void</p> |

**示例**  
```js
client.kick('kicked_user', 'is_admin', 'kick_time', (uid) => {}, (err) => {})
```
<a name="RTCClient+leave"></a>

### rtcClient.leave([onSuccess], [onFailure]) ⇒ <code>void</code>
<p>离开频道
调用该方法离开频道时，本地会触发 Client.on(&quot;connection-state-change&quot;) 回调；
通信场景下的用户和直播场景下的主播离开频道后，远端会触发 Client.on(&quot;peer-leave&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| [onSuccess] | <code>function</code> | <p>可选；成功回调；() =&gt; void;</p> |
| [onFailure] | <code>function</code> | <p>可选；失败回调；(err: string) =&gt; void;</p> |

**示例**  
```js
client.leave(() => {}, (err) => {});
```
<a name="RTCClient+createStream"></a>

### rtcClient.createStream(spec) ⇒ [<code>Stream</code>](#Stream)
<p>该方法创建并返回音视频流对象。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  
**返回值**: [<code>Stream</code>](#Stream) - <ul>
<li>流对象</li>
</ul>  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| spec | [<code>StreamSpec</code>](#StreamSpec) | <p>定义音视频流对象的属性，详见 {StreamSpec}。</p> |

<a name="RTCClient+publish"></a>

### rtcClient.publish(stream, [onSuccess], onFailure) ⇒ <code>void</code>
<p>发布本地音视频流
发布音视频流之后，本地会触发 client.on(&quot;stream-published&quot;) 回调；远端会触发 client.on(&quot;stream-added&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| stream | [<code>Stream</code>](#Stream) | <p>要被发布的本地音视频流对象</p> |
| [onSuccess] | <code>function</code> | <p>可选；成功回调；() =&gt; void;</p> |
| onFailure | <code>function</code> | <p>可选；失败回调；(err: string) =&gt; void</p> |

**示例**  
```js
client.publish(stream, (err) => {});
```
<a name="RTCClient+unpublish"></a>

### rtcClient.unpublish(stream, [onSuccess], [onFailure]) ⇒ <code>void</code>
<p>取消发布本地音视频流
该方法取消发布本地音视频流。
取消发布音视频流之后，本地会触发 client.on(&quot;stream-unpublished&quot;) 回调，远端会触发 client.on(&quot;stream-removed&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| stream | [<code>Stream</code>](#Stream) | <p>要取消发布的本地音视频流对象</p> |
| [onSuccess] | <code>function</code> | <p>可选；成功回调；() =&gt; void;</p> |
| [onFailure] | <code>function</code> | <p>可选；失败回调；(err: string) =&gt; void;</p> |

**示例**  
```js
client.unpublish(stream, (err) => {});
```
<a name="RTCClient+subscribe"></a>

### rtcClient.subscribe(stream, [options], [onSuccess], [onFailure])
<p>订阅远端音视频流
该方法从服务器端接收远端音视频流。
订阅远端音视频流之后，本地会触发 client.on(&quot;stream-subscribed&quot;) 回调。
如果订阅流中包含音频，还会触发 client.on(&quot;first-audio-frame-decode&quot;) 回调；- 待实现
如果订阅流中包含视频，还会触发 client.on(&quot;first-video-frame-decode&quot;) 回调。- 待实现</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| stream | [<code>Stream</code>](#Stream) | <p>要取消订阅的远端音视频流</p> |
| [options] | [<code>SubscribeOptions</code>](#SubscribeOptions) | <p>可选；选项参数；默认订阅音视频流</p> |
| [onSuccess] | <code>function</code> | <p>可选；成功回调；() =&gt; void;</p> |
| [onFailure] | <code>function</code> | <p>可选；失败回调；(err) =&gt; void;</p> |

**示例**  
```js
// 基本用法
client.subscribe(stream, (err) => {});
```
**示例**  
```js
// 高级用法
client.subscribe(stream, { video: true, audio: true }, (err) => {});
```
<a name="RTCClient+unsubscribe"></a>

### rtcClient.unsubscribe(stream, [onSuccess], [onFailure]) ⇒ <code>void</code>
<p>取消订阅远端音视频流
该方法取消接收远端音视频流。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| stream | [<code>Stream</code>](#Stream) | <p>要取消订阅的远端流</p> |
| [onSuccess] | <code>function</code> | <p>可选；成功回调；() =&gt; void;</p> |
| [onFailure] | <code>function</code> | <p>可选；失败回调；(err: string) =&gt; void; 方法调用失败时执行的回调函数，常见的错误如下：</p> <ul> <li>&quot;INVALID_REMOTE_STREAM&quot;: 传入的 stream 对象格式非法。</li> <li>&quot;INVALID_OPERATION&quot;: 当前不在频道中，可能是没有加入或者是网络波动导致暂时断开连接。</li> <li>&quot;NO_SUCH_REMOTE_STREAM&quot;: 没有找到要取消订阅的远端流。</li> <li>&quot;UNSUBSCRIBE_STREAM_FAILED&quot;: 取消订阅失败，一般是取消订阅时连接断开。</li> </ul> |

**示例**  
```js
client.unsubscribe(stream, ()=>{},(err) => {});
```
<a name="RTCClient+setClientRole"></a>

### rtcClient.setClientRole(role, [callback]) ⇒ <code>void</code>
<p>设置用户角色
本方法仅适用于直播场景。
直播场景下，可以调用本方法设置用户角色。
在加入频道前，用户可以通过本方法设置自己的角色。
在加入频道后，用户可以通过本方法切换角色：
用户在调用 publish 的时候会自动切换成 host 角色，调用 unpublish 的时候会自动切换为 audience 角色。
如果已经调用了 publish，调用本方法将用户角色设置为 audience，会自动进行 unpublish。
如果你在加入频道后调用该方法切换用户角色，切换成功后，本地会触发 client.on(&quot;client-role-changed&quot;) 回调；远端会触发 client.on(&quot;peer-online&quot;) 或者 client.on(&quot;peer-leave&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| role | <code>string</code> | <p>角色, 'host'或'audience'</p> |
| [callback] | <code>function</code> | <p>可选；回调；(err?: string | null) =&gt; void;</p> |

**示例**  
```js
client.setClientRole('host', (err) => {
    if (err) {
        // 失败
    } else {
        // 成功
    }
});
```
<a name="RTCClient+sendMessage"></a>

### rtcClient.sendMessage()
<p>发送自定义消息，支持和原生SDK的sendMessage 接口互通</p>
<p>接收者通过 client.on(&quot;custom-message&quot;) 接收
user_id 为可选参数，用于指定特定的接收userid，如果不指定就是频道内广播</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  
<a name="RTCClient+startLiveStreaming"></a>

### rtcClient.startLiveStreaming(url) ⇒ <code>void</code>
<p>新建直播流
该方法新建直播流
推流开始后本地会触发 client.on(&quot;liveStreamingStarted&quot;) 回调。如果推流失败会触发 client.on(&quot;liveStreamingFailed&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 描述 |
| --- | --- |
| url | <p>接收推流的地址例如: rtmp://xxxxxxxxx</p> |

**示例**  
```js
client.setLiveTranscoding(<coding>);
client.startLiveStreaming(<url>)
```
<a name="RTCClient+setLiveTranscoding"></a>

### rtcClient.setLiveTranscoding(coding) ⇒ <code>void</code>
<p>设置直播推流转码
调用该方法更新转码设置后本地会触发 client.on(&quot;liveTranscodingUpdated&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| coding | [<code>LiveTranscoding</code>](#LiveTranscoding) | <p>转码设置</p> |

<a name="RTCClient+stopLiveStreaming"></a>

### rtcClient.stopLiveStreaming(url) ⇒ <code>void</code>
<p>删除直播流
该方法停止并删除直播流。推流停止后本地会触发 client.on(&quot;liveStreamingStopped&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| url | <code>string</code> | <p>要停止的流url</p> |

<a name="RTCClient+getNetworkStats"></a>

### ~~rtcClient.getNetworkStats(callback)~~
***已废弃***

<p>获取系统网络信息</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| callback | <code>function</code> | <p>回调 (stats: NetworkStats) =&gt; void</p> |

**示例**  
```js
client.getNetworkStats((stats) => {
  console.log(`Current network type: ${stats.NetworkType}`);
});
```
<a name="RTCClient+getRecordingDevices"></a>

### rtcClient.getRecordingDevices(callback) ⇒ <code>void</code>
<p>枚举音频输入设备
该方法枚举可用的音频输入设备，比如麦克风。
调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的音频输入设备。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| callback | <code>function</code> | <p>(devices: MediaDeviceInfo[]): void</p> |

**示例**  
```js
client.getRecordingDevices((devices) => {});
```
<a name="RTCClient+getPlayoutDevices"></a>

### rtcClient.getPlayoutDevices(callback) ⇒ <code>void</code>
<p>枚举音频输出设备</p>
<p>该方法枚举可用的音频输出设备，比如扬声器。</p>
<p>调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的音频输出设备。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| callback | <code>function</code> | <p>回调 (devices: MediaDeviceInfo[]) =&gt; void</p> |

<a name="RTCClient+getCameras"></a>

### rtcClient.getCameras(callback) ⇒ <code>void</code>
<p>枚举视频输入设备
该方法枚举可用的视频输入设备，比如摄像头。
调用成功后 SDK 会通过 MediaDeviceInfo 对象返回可用的视频输入设备。</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 |
| --- | --- |
| callback | [<code>getCameras</code>](#getCameras) | 

<a name="RTCClient+setRePubAndSub"></a>

### rtcClient.setRePubAndSub(flag) ⇒ <code>void</code>
<p>设置断网重连后是否重新发布及订阅</p>

**类型**: 实例方法(function)，来自 [<code>RTCClient</code>](#RTCClient)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| flag | <code>0</code> \| <code>1</code> | <p>是否重新发布及订阅标志，默认是1，如果设置为0则需要业务自己去做重连操作</p> |

**示例**  
```js
client.setRePubAndSub(0);
```
<a name="RTCClient+event_stream-added"></a>

### "event:stream-added"
<p>该回调通知应用远端音视频流已添加。
如果用户加入频道时频道内已经有其他用户，也会收到该回调报告已经存在的远端流。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型, 为 <code>stream-added</code></p> |
| msg | <code>string</code> | <p>消息</p> |
| reason | <code>string</code> | <p>原因</p> |
| stream | [<code>Stream</code>](#Stream) | <p>远端流</p> |

<a name="RTCClient+event_stream-removed"></a>

### "event:stream-removed"
<p>该回调通知应用已删除远端音视频流，即对方调用了 <code>client.unpublish</code>。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，为 <code>stream-removed</code></p> |
| msg | <code>string</code> | <p>消息</p> |
| reason | <code>string</code> | <p>原因</p> |
| stream | [<code>Stream</code>](#Stream) | <p>远端流</p> |

<a name="RTCClient+event_peer-online"></a>

### "event:peer-online"
<p>该回调提示有远端用户/主播加入频道。</p>
<ul>
<li>通信场景下，该回调提示有远端用户加入了频道，并返回新加入用户的 ID；如果加入之前，已经有其他用户在频道中了，新加入的用户也会收到这些已有用户加入频道的回调。</li>
<li>直播场景下，该回调提示有主播加入了频道，并返回该主播的 ID。如果在加入之前，已经有主播在频道中了，新加入的用户也会收到已有主播加入频道的回调。建议连麦主播不超过 20</li>
</ul>
<p>该回调在如下情况下会被触发：</p>
<ul>
<li>通信场景的远端用户/直播场景的远端主播调用 client.join 方法加入频道。</li>
<li>直播场景的远端观众加入频道后调用 client.setClientRole 将用户角色改变为主播。</li>
</ul>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户ID</p> |

<a name="RTCClient+event_peer-leave"></a>

### "event:peer-leave"
<p>该回调通知应用有远端用户离线。
在直播场景中，只有角色为主播的用户会触发该回调。</p>
<p>离线包括以下情况：</p>
<ul>
<li>用户离开频道
<ul>
<li>正常离开：用户会收到类似“再见”的消息，接收此消息后，判断对方离开频道。</li>
<li>超时掉线：在 20 秒内，用户没有收到对方的任何数据包，则判定为对方掉线。在网络较差的情况下，有可能会误报。建议使用信令系统来做可靠的掉线检测。</li>
</ul>
</li>
<li>用户角色从主播变为观众</li>
</ul>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| reason | <code>string</code> | <p>用户离线原因</p> <ul> <li>&quot;Quit&quot;: 用户调用了 Client.leave 主动离开频道。</li> <li>&quot;BecomeAudience&quot;: 用户角色从主播切换为观众。</li> </ul> |

**示例**  
```js
client.on("peer-leave", function(evt) {
  var uid = evt.uid;
  var reason = evt.reason;
  console.log("remote user left ", uid, "reason: ", reason);
});
```
<a name="RTCClient+event_request-unmute-audio"></a>

### "event:request-unmute-audio"
<p>远端用户求情把麦克风声音开启，收到该请求需要自己调用接口取消麦克风静音。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户id</p> |

**示例**  
```js
client.on("request-unmute-audio", function(evt) {
   var uid = evt.uid;
   console.log(uid + "request unmute my audio" );
});
```
<a name="RTCClient+event_request-mute-audio"></a>

### "event:request-mute-audio"
<p>远端用户求情把麦克风声音关闭，收到该请求需要自己调用接口静音麦克风。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户id</p> |

**示例**  
```js
client.on("request-mute-audio", function(evt) {
   var uid = evt.uid;
   console.log(uid + "request mute my audio");
});
```
<a name="RTCClient+event_unmute-audio"></a>

### "event:unmute-audio"
<p>该回调通知应用对方用户在语音通话中将自己的声音打开。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户id</p> |

**示例**  
```js
client.on("unmute-audio", function(evt) {
   var uid = evt.uid;
   console.log("unmute audio:" + uid);
});
```
<a name="RTCClient+event_mute-audio"></a>

### "event:mute-audio"
<p>该回调通知应用对方用户在语音通话中将自己的声音关闭。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户id</p> |

**示例**  
```js
client.on("mute-audio", function(evt) {
   var uid = evt.uid;
   console.log("mute audio:" + uid);
});
```
<a name="RTCClient+event_unmute-video"></a>

### "event:unmute-video"
<p>该回调通知应用对方用户在语音通话中将自己的视频打开。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户id</p> |

**示例**  
```js
client.on("unmute-video", function(evt) {
   var uid = evt.uid;
   console.log("unmute video:" + uid);
});
```
<a name="RTCClient+event_mute-video"></a>

### "event:mute-video"
<p>该回调通知应用对方用户在语音通话中将自己的视频关闭。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| uid | <code>string</code> | <p>用户id</p> |

**示例**  
```js
client.on("mute-video", function(evt) {
   var uid = evt.uid;
   console.log("mute video:" + uid);
});
```
<a name="RTCClient+event_client-role-changed"></a>

### "event:client-role-changed"
<p>该回调通知本地用户的角色发生改变。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 类型 | 描述 |
| --- | --- |
| <code>role</code> | <p>角色，'host'或'audience'</p> |

**示例**  
```js
client.on("client-role-changed", function(evt) {
    console.log("client-role-changed", evt.role);
});
```
<a name="RTCClient+event_stream-published"></a>

### "event:stream-published"
<p>该回调通知应用本地音视频流已发布。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型, 值为<code>stream-published</code></p> |
| proxy | <code>string</code> | <p>是否为代理</p> |
| stream | [<code>Stream</code>](#Stream) | <p>本地流</p> |

**示例**  
```js
client.on("stream-published", function(evt) {
  console.log('local stream published');
  //……
});
```
<a name="RTCClient+event_stream-unpublished"></a>

### "event:stream-unpublished"
<p>该回调通知应用本地音视频流已取消发布。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>stream-unpublished</code></p> |
| stream | [<code>Stream</code>](#Stream) | <p>本地流</p> |

**示例**  
```js
client.on("stream-unpublished", function(evt) {
  console.log('local stream unpublished');
  //……
});
```
<a name="RTCClient+event_stream-subscribed"></a>

### "event:stream-subscribed"
<p>该回调通知应用已接收远端音视频流。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>stream-subscribed</code></p> |
| msg | <code>string</code> | <p>消息</p> |
| reason | <code>string</code> | <p>原因</p> |
| stream | [<code>Stream</code>](#Stream) | <p>远端流</p> |

**示例**  
```js
client.on("stream-subscribed", function(evt) {
  var stream = evt.stream;
  console.log("new stream subscribed ", stream.getId());
  // Play the stream.
  //……
});
```
<a name="RTCClient+event_stream-updated"></a>

### "event:stream-updated"
<p>该回调表示订阅流的音视频轨道发生变化。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>stream-updated</code></p> |
| stream | [<code>Stream</code>](#Stream) | <p>远端流</p> |
| video | <code>boolean</code> | <p>是否有视频</p> |
| audio | <code>boolean</code> | <p>是否有音频</p> |

<a name="RTCClient+event_media-stats-update"></a>

### "event:media-stats-update"
<p>该回调表示远端流的网络统计情况，仅 Chromium/Chrome 内核支持比较完整</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>media-stats-update</code></p> |
| stream | [<code>Stream</code>](#Stream) | <p>流</p> |
| statObj | <code>RemoteStreamStats</code> | <p>统计结果</p> |
| 详细字段 | <code>RemoteStreamStats</code> |  |
| : | <code>audioReceiveBytes</code> | <p>Bytes of the received audio.</p> |
| : | <code>audioReceivePackets</code> | <p>Number of packets of the received audio.</p> |
| : | <code>audioReceivePacketsLost</code> | <p>Number of lost packets of the received audio.</p> |
| : | <code>videoReceiveBytes</code> | <p>Bytes of the received video</p> |
| : | <code>videoReceivePackets</code> | <p>Packets of the received video</p> |
| : | <code>videoReceivePacketsLost</code> | <p>Number of lost packets of the received video</p> |
| : | <code>videoReceiveResolutionWidth</code> | <p>Resolution width of the received video</p> |
| : | <code>videoReceiveResolutionHeight</code> | <p>Resolution height of the received video</p> |
| : | <code>videoReceiveDelay</code> | <p>Delay in receiving the video (ms).</p> |
| : | <code>audioReceiveDelay</code> | <p>Delay in receiving the audio (ms).</p> |

<a name="RTCClient+event_local-media-stats-update"></a>

### "event:local-media-stats-update"
<p>该回调表示本地流的网络统计情况，仅 Chromium/Chrome 内核支持比较完整</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>media-stats-update</code></p> |
| statObj | <code>LocalStreamStats</code> | <p>统计结果</p> |
| 详细字段 | <code>LocalStreamStats</code> |  |
| : | <code>audioSendBytes</code> | <p>Bytes of the sent audio.</p> |
| : | <code>audioSendPackets</code> | <p>Number of packets of the sent audio.</p> |
| : | <code>audioSendPacketsLost</code> | <p>Number of lost packets of the sent audio.</p> |
| : | <code>audioRtt</code> | <p>Rtt of the sent audio.</p> |
| : | <code>videoSendBytes</code> | <p>Bytes of the sent video</p> |
| : | <code>videoSendPackets</code> | <p>Packets of the sent video</p> |
| : | <code>videoSendPacketsLost</code> | <p>Number of lost packets of the sent video</p> |
| : | <code>videoSendResolutionWidth</code> | <p>Resolution width of the sent video</p> |
| : | <code>videoSendResolutionHeight</code> | <p>Resolution height of the sent video</p> |
| : | <code>accessDelay</code> | <p>Delay in sent (ms).</p> |

<a name="RTCClient+event_connection-state-change"></a>

### "event:connection-state-change"
<p>该回调通知应用SDK 与服务器的连接状态发生改变</p>
<p>SDK 与服务器的连接状态共有以下 4 种：</p>
<ul>
<li>DISCONNECTED：连接断开。该状态表示 SDK 处于以下任一阶段：
<ul>
<li>调用 <code>client.join</code> 加入频道前的初始化阶段</li>
<li>调用 <code>client.leave</code> 后的离开频道阶段</li>
</ul>
</li>
<li>CONNECTING：正在连接中。在调用 <code>client.join</code> 或者连接中断自动重连的时候为此状态。</li>
<li>CONNECTED：已连接。该状态表示用户已经加入频道，可以在频道内发布或订阅媒体流。如果因
网络断开或切换而导致 SDK 与服务器的连接中断，SDK 会自动重连，此时应用会收到该回调，通知网络状态由 CONNECTED 变为 CONNECTING。</li>
<li>DISCONNECTING：正在断开连接。在调用 Client.leave 的时候为此状态。</li>
</ul>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| curState | <code>string</code> | <p>当前的状态</p> |

**示例**  
```js
client.on("connection-state-change", function(evt) {
    console.log(evt.curState);
});
```
<a name="RTCClient+event_reconnect"></a>

### "event:reconnect"
<p>该回调通知正在与服务器重新建立连接。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>reconnect</code></p> |

**示例**  
```js
client.on("reconnect", function() {
    console.log('reconnect');
});
```
<a name="RTCClient+event_connected"></a>

### "event:connected"
<p>该回调通知与服务器已经成功建立连接。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>connected</code></p> |

**示例**  
```js
client.on("connected", function() {
   console.log('connected');
});
```
<a name="RTCClient+event_error"></a>

### "event:error"
<p>该回调通知应用有错误信息，需要进行处理，可能的错误如下：</p>
<ul>
<li>reason 为 &quot;SOCKET_DISCONNECTED&quot; 表示此时 SDK 重连失败，需要重新进入频道。</li>
<li>reason 为其他错误，表示在连接断开后 SDK 自动重连阶段触发的错误。</li>
</ul>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>error</code></p> |

**示例**  
```js
client.on("error", function(err) {
  console.log("Got error msg:", err.reason);
});
```
<a name="RTCClient+event_kicked-other-user"></a>

### "event:kicked-other-user"
<p>该回调表示用户踢人事件回调通知（非本人被踢）。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>kicked-other-user</code></p> |
| uid | <code>string</code> | <p>被踢人id</p> |

<a name="RTCClient+event_kicked-user"></a>

### "event:kicked-user"
<p>该回调表示用户踢人事件回调通知（本人被踢）。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>kicked-user</code></p> |
| uid | <code>string</code> | <p>被踢人id</p> |

<a name="RTCClient+event_kickout-failed"></a>

### "event:kickout-failed"
<p>该回调表示用户踢人失败事件回调通知。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>kickout-failed</code></p> |
| uid | <code>string</code> | <p>被踢人id</p> |
| err_code | <code>number</code> | <p>失败错误码，450:不能踢出本人,1006: 用户不在频道</p> |
| err_msg | <code>string</code> | <p>失败错误信息</p> |

<a name="RTCClient+event_joined"></a>

### "event:joined"
<p>该回调表示用户加入房间成功事件回调通知。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>joined</code></p> |
| uid | <code>string</code> | <p>用户id</p> |
| room | <code>string</code> | <p>房间id</p> |

<a name="RTCClient+event_joinroom_failed"></a>

### "event:joinroom_failed"
<p>该回调表示用户加入房间失败事件回调通知。</p>

**类型**: 事件，由 [<code>RTCClient</code>](#RTCClient) 触发  
**属性**

| 名称 | 类型 | 描述 |
| --- | --- | --- |
| type | <code>string</code> | <p>事件类型，值为<code>joinroom_failed</code></p> |
| uid | <code>string</code> | <p>用户id</p> |
| room | <code>string</code> | <p>房间id</p> |
| err_code | <code>number</code> | <p>失败错误码，1001:客户端传入的token不正确,1007: 用户频道黑名单中</p> |
| err_msg | <code>string</code> | <p>失败错误信息</p> |

<a name="Stream"></a>

## Stream
<p>流对象</p>

**类型**: 全局类(class)  

* [Stream](#Stream)
    * [.resume()](#Stream+resume)
    * [.stop()](#Stream+stop) ⇒ <code>void</code>
    * [.isPlaying()](#Stream+isPlaying) ⇒ <code>boolean</code>
    * [.unmuteAudio()](#Stream+unmuteAudio) ⇒ <code>boolean</code>
    * [.muteAudio()](#Stream+muteAudio) ⇒ <code>boolean</code>
    * [.unmuteVideo()](#Stream+unmuteVideo) ⇒ <code>boolean</code>
    * [.muteVideo()](#Stream+muteVideo) ⇒
    * ~~[.enableAudio()](#Stream+enableAudio) ⇒ <code>boolean</code>~~
    * ~~[.enableVideo()](#Stream+enableVideo) ⇒ <code>boolean</code>~~
    * [.hasAudio()](#Stream+hasAudio) ⇒ <code>boolean</code>
    * [.hasVideo()](#Stream+hasVideo) ⇒ <code>boolean</code>
    * [.getAudioTrack()](#Stream+getAudioTrack) ⇒ <code>MediaStreamTrack</code> \| <code>undefined</code>
    * [.getVideoTrack()](#Stream+getVideoTrack) ⇒ <code>MediaStreamTrack</code> \| <code>undefined</code>
    * [.getTracks()](#Stream+getTracks) ⇒ <code>Array.&lt;MediaStreamTrack&gt;</code> \| <code>undefined</code>
    * [.setAudioVolume(track, [onSuccess], [onFailure])](#Stream+setAudioVolume)
    * [.setScreenProfile(profile)](#Stream+setScreenProfile)
    * [.getId()](#Stream+getId) ⇒ <code>string</code>

<a name="Stream+resume"></a>

### stream.resume()
<p>恢复音视频流播放
该方法一般在调用 Stream.play 之后，播放失败时调用。通常播放失败是由于浏览器策略阻止导致的。
该方法需用手势触发。详见处理浏览器的自动播放策略。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**完成回调**: <code>any</code> - 恢复成功  
**拒绝回调**: <code>any</code> - 恢复失败  
<a name="Stream+stop"></a>

### stream.stop() ⇒ <code>void</code>
<p>停止音视频流
调用该方法停止播放 <code>Stream.play</code> 播放的音视频流。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
<a name="Stream+isPlaying"></a>

### stream.isPlaying() ⇒ <code>boolean</code>
<p>返回音视频流当前是否在播放状态</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true：该音视频流正在渲染或播放。</li>
<li>false：该音视频流没有渲染。</li>
</ul>  
<a name="Stream+unmuteAudio"></a>

### stream.unmuteAudio() ⇒ <code>boolean</code>
<p>启用音频轨道
该方法启用音频轨道。对本地流启用音频轨道后远端会触发 client.on(&quot;unmute-audio&quot;) 回调。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true：启用音频轨道成功。</li>
<li>false：启用音频轨道失败，可能的原因包括没有音频、流未初始化、音频轨道已经启用等。</li>
</ul>  
<a name="Stream+muteAudio"></a>

### stream.muteAudio() ⇒ <code>boolean</code>
<p>禁用音频轨道
该方法禁用音频轨道。
对于本地流，调用该方法会停止发送音频，远端会触发 Client.on(&quot;mute-audio&quot;) 回调。
对于远端流，调用该方法仍然会接收音频，但是会停止播放。
对于本地流，在 createStream 时将 audio 设置为 true 才可使用该方法。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true：禁用音频轨道成功。</li>
<li>false：禁用音频轨道失败，可能的原因包括没有音频、流未初始化、音频轨道已经禁用等。</li>
</ul>  
<a name="Stream+unmuteVideo"></a>

### stream.unmuteVideo() ⇒ <code>boolean</code>
<p>启用视频轨道
该方法启用视频轨道。对本地流启用视频轨道后远端会触发 Client.on(&quot;unmute-video&quot;) 回调。
Note
对于本地创建的流，在 createStream 时将 video 设置为 true 才可使用该方法。
视频轨道默认为开启状态。如果你调用了 muteVideo，可调用本方法启用视频。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true：启用视频轨道成功。</li>
<li>false：启用视频轨道失败，可能的原因包括没有视频、流未初始化、视频轨道已经启用等。</li>
</ul>  
<a name="Stream+muteVideo"></a>

### stream.muteVideo() ⇒
<p>禁用视频轨道
该方法禁用视频轨道。
对于本地流，调用该方法会停止发送视频，远端会触发 client.on(&quot;mute-video&quot;) 回调。
对于远端流，调用该方法仍然会接收视频，但是会停止播放。
对于本地创建的流，在 createStream 时将 video 设置为 true 才可使用该方法。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <ul>
<li>true：禁用视频轨道成功。</li>
<li>false：禁用视频轨道失败，可能的原因包括没有视频、流未初始化、视频轨道已经禁用等。</li>
</ul>  
<a name="Stream+enableAudio"></a>

### ~~stream.enableAudio() ⇒ <code>boolean</code>~~
***已废弃***

<p>启用音频轨道
请使用 unmuteAudio。
该方法启用音频轨道。在 createStream 时将 audio 设置为 true 才可使用该方法。
音频轨道默认为开启状态。如果你调用了 disableAudio，可调用本方法启用音频。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true：启用音频轨道成功。</li>
<li>false：启用音频轨道失败，可能的原因包括没有音频、流未初始化、音频轨道已经启用等。</li>
</ul>  
<a name="Stream+enableVideo"></a>

### ~~stream.enableVideo() ⇒ <code>boolean</code>~~
***已废弃***

<p>启用视频轨道
请使用 unmuteVideo。
该方法启用视频轨道。在 createStream 时将 video 设置为 true 才可使用该方法。
视频轨道默认为开启状态。如果你调用了 disableVideo，可调用本方法启用视频。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true：启用视频轨道成功。</li>
<li>false：启用视频轨道失败，可能的原因包括没有视频、流未初始化、视频轨道已经启用等。</li>
</ul>  
<a name="Stream+hasAudio"></a>

### stream.hasAudio() ⇒ <code>boolean</code>
<p>获取音频 flag
该方法仅对本地流有效。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true: 该音视频流对象中包含音频资源。</li>
<li>false: 该音视频流对象中不包含音频资源。</li>
</ul>  
<a name="Stream+hasVideo"></a>

### stream.hasVideo() ⇒ <code>boolean</code>
<p>获取视频 flag
该方法仅对本地流有效。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>boolean</code> - <ul>
<li>true: 该音视频流对象中包含视频资源。</li>
<li>false: 该音视频流对象中不包含视频资源。</li>
</ul>  
<a name="Stream+getAudioTrack"></a>

### stream.getAudioTrack() ⇒ <code>MediaStreamTrack</code> \| <code>undefined</code>
<p>获取音频轨道
该方法获取音视频流中的音频轨道，可与 replaceTrack 搭配使用。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>MediaStreamTrack</code> \| <code>undefined</code> - <ul>
<li>如果音视频流中包含音频轨道，会以 MediaStreamTrack 对象返回。</li>
</ul>  
<a name="Stream+getVideoTrack"></a>

### stream.getVideoTrack() ⇒ <code>MediaStreamTrack</code> \| <code>undefined</code>
<p>获取视频轨道
该方法获取音视频流中的视频轨道，可与 replaceTrack 搭配使用。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>MediaStreamTrack</code> \| <code>undefined</code> - <ul>
<li>如果音视频流中包含视频轨道，会以 MediaStreamTrack 对象返回。</li>
</ul>  
<a name="Stream+getTracks"></a>

### stream.getTracks() ⇒ <code>Array.&lt;MediaStreamTrack&gt;</code> \| <code>undefined</code>
<p>获取音视频轨道</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>Array.&lt;MediaStreamTrack&gt;</code> \| <code>undefined</code> - <ul>
<li>如果音视频流存在且包含音频或视频流，会以 MediaStreamTrack数组返回</li>
</ul>  
<a name="Stream+setAudioVolume"></a>

### stream.setAudioVolume(track, [onSuccess], [onFailure])
<p>替换音视频轨道
该方法可以替换本地音视频流中的音视频轨道。
本地流发布后，可以通过该方法切换摄像头或者切换麦克风和播放的 mp3 等。
新的音视频轨道可以通过 getUserMedia，MediaElement.captureStream 等方法获取。
被替换的音视频轨道会被强制停止。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| track | <code>MediaStreamTrack</code> | <p>要替换的track</p> |
| [onSuccess] | <code>function</code> | <p>可选；成功回调；() =&gt; void;</p> |
| [onFailure] | <code>function</code> | <ul> <li>可选；失败回调；(err: string) =&gt; void;</li> <li>方法调用失败的回调。常见的错误如下：</li> <li>&quot;INVALID_TRACK&quot; 和 &quot;INVALID_TRACK_TYPE&quot;: 新传入的 MediaStreamTrack 对象无法识别，请检查传入对象。</li> <li>&quot;MEDIASTREAM_TRACK_NOT_FOUND&quot;: 找不到替换对象，比如在纯音频流上尝试替换一个视频的 Track。</li> <li>&quot;NO_STREAM_FOUND&quot;: 找不到本地流对象，无法替换。</li> </ul> |

**示例**  
```js
- 支持 Chrome 65+，Safari 以及最新版 Firefox 浏览器。
- 部分移动设备上该方法可能不生效。
- Firefox 不支持在不同的麦克风之间切换音频轨道，可以支持切换麦克风和播放的音乐文件。
- Safari 部分外接音频设备不支持此方法。
- 订阅端无法知悉音视频轨道被替换。
- 推荐使用 switchDevice 方法来切换媒体输入设备。
```
<a name="Stream+setScreenProfile"></a>

### stream.setScreenProfile(profile)
<p>设置屏幕属性
该方法设置屏幕共享时屏幕的显示属性，必须在 Stream.init 之前调用。</p>
<ul>
<li>该方法仅可对本地流调用。</li>
</ul>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  

| 参数 |
| --- |
| profile | 

<a name="Stream+getId"></a>

### stream.getId() ⇒ <code>string</code>
<p>获取音视频流 ID
该方法可以获取音视频流 ID。</p>

**类型**: 实例方法(function)，来自 [<code>Stream</code>](#Stream)  
**返回值**: <code>string</code> - <ul>
<li>流id</li>
</ul>  
<a name="ClientConfig"></a>

## ClientConfig : <code>Object</code>
<p>定义 createClient 中的 config 参数的接口。</p>

**类型**: 全局自定义类型(typedef)  

| 参数 | 类型 | 默认值 | 描述 |
| --- | --- | --- | --- |
| mode | <code>string</code> |  | <p>模式: 'rtc'或'live'</p> |
| codec | <code>string</code> |  | <p>编码方式： 'vp8','h264'或'h264,vp8', 推荐默认填'h264,vp8'，兼容性最好</p> |
| server | <code>string</code> |  | <p>服务器地址，比如：'wss://xxxx.com'，需与youme确认是否支持</p> |
| [debug] | <code>boolean</code> | <code>false</code> | <p>是否开启调试模式，调试模式会输出内部日志；默认false</p> |

<a name="StreamSpec"></a>

## StreamSpec : <code>Object</code>
<p>创建流的配置参数</p>

**类型**: 全局自定义类型(typedef)  

| 参数 | 类型 | 默认值 | 描述 |
| --- | --- | --- | --- |
| [streamID] | <code>number</code> \| <code>string</code> | <code>&#x27;&#x27;</code> | <p>请设置streamId 为用户ID, 用户id可以从[RTCClient.join](RTCClient.join)返回值获取到</p> |
| audio | <code>boolean</code> |  | <p>表示流中是否包含音频轨道</p> |
| video | <code>boolean</code> |  | <p>表示流中是否包含视频轨道</p> |
| [screen] | <code>boolean</code> |  | <p>表示是否包含屏幕共享轨道</p> |
| [screenAudio] | <code>boolean</code> | <code>false</code> | <p><code>true</code>: 共享时共享本地正在播放的声音 <code>false</code>: 默认值，共享时不要共享本地播放的声音</p> |

<a name="GetDevicesSuccCb"></a>

## GetDevicesSuccCb ⇒ <code>void</code>
<p>getDevices 成功回调</p>

**类型**: 全局自定义类型(typedef)  

| 类型 | 描述 |
| --- | --- |
| <code>Array.&lt;MediaDeviceInfo&gt;</code> | <p>devices 设备类型</p> |

<a name="GetDevicesFailCb"></a>

## GetDevicesFailCb ⇒ <code>void</code>
<p>getDevices 失败回调</p>

**类型**: 全局自定义类型(typedef)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| errStr | <code>string</code> | <p>错误原因</p> |

<a name="LiveTranscoding"></a>

## LiveTranscoding : <code>Object</code>
<p>管理 CDN 直播推流转码的接口
调用 <code>setLiveTranscoding</code> 时可以通过该接口设置直播推流。</p>

**类型**: 全局自定义类型(typedef)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| [width] | <code>number</code> | <p>视频的宽</p> |
| [height] | <code>number</code> | <p>视频的高</p> |
| [videoBitrate] | <code>number</code> | <p>CDN直播流的视频码率（Kbps）</p> |
| [videoFramerate] | <code>number</code> | <p>CDN直播流的帧率（fps）</p> |
| [audioSampleRate] | <code>number</code> | <p>音频采样率 32000： 32kHz, 44100: (默认） 44.1kHz, 48000: 48kHz</p> |
| [audioBitrate] | <code>number</code> | <p>CDN直播流的音频码率（Kbps）</p> |
| [audioChannels] | <code>number</code> | <p>CDN直播流的声道数. 1: (默认)Mono, 2: 立体, 3, 4, 5声道</p> |

<a name="NetworkType"></a>

## NetworkType : <code>Object</code>
<p>网络类型</p>
<ul>
<li>&quot;bluetooth&quot;：蓝牙网络</li>
<li>&quot;cellular&quot;：蜂窝移动数据网络</li>
<li>&quot;ethernet&quot;：以太网</li>
<li>&quot;none&quot;：没有网络</li>
<li>&quot;wifi&quot;：Wi-Fi</li>
<li>&quot;wimax&quot;：WiMax</li>
<li>&quot;other&quot;：其他网络类型</li>
<li>&quot;unknown&quot;：未知网络类型</li>
<li>&quot;UNSUPPORTED&quot;：浏览器不支持获取网络类型</li>
</ul>
<p>该功能仅支持 Chrome 61 及以上版本，且无法保证兼容性</p>

**类型**: 全局自定义类型(typedef)  
<a name="SubscribeOptions"></a>

## SubscribeOptions : <code>Object</code>
<p>subscribe方法需要的options参数</p>

**类型**: 全局自定义类型(typedef)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| [video] | <code>boolean</code> | <p>可选；是否订阅视频</p> |
| [audio] | <code>boolean</code> | <p>可选；是否订阅音频</p> |

<a name="getCameras"></a>

## getCameras ⇒ <code>void</code>
<p>getCameras方法回调函数类型</p>

**类型**: 全局自定义类型(typedef)  

| 参数 | 类型 | 描述 |
| --- | --- | --- |
| devices | <code>Array.&lt;MediaDeviceInfo&gt;</code> | <p>视频输入设备列表</p> |

