/**
 * @file 游密 WebRTC 演示代码
 * @author BenzLeung(https://github.com/BenzLeung)
 * @date 2018/1/21
 * Created by JetBrains PhpStorm.
 *
 * 每位工程师都有保持代码优雅的义务
 * each engineer has a duty to keep the code elegant
 */

 (function () {
    const YM_APPKEY = 'YOUME20859B1EE41178CCEA2B6528073C74F16CE6652C';
    const APIKEY = 'b457938bb1840eb73d1118902ee91610'; // 实际使用时，API KEY只需要放到服务器端，用于token计算
    const APPID = 6976;

    function E(id) {
        return document.getElementById(id);
    }

    console.log('version', YMRTC.VERSION);

    // ==================== Token 计算演示 ======================
    function genToken(appkey, apikey, userid) {
        const sha1src = appkey + apikey + userid;
        return CryptoJS.SHA1(sha1src).toString(CryptoJS.enc.Hex);
    }

    // ==================== TokenV3 计算演示 ======================
    function genTokenV3(appkey, apikey, roomid, userid, timestamp) {
        //tokenV3计算方式：token=sha1(apikey+appkey+roomid+userid+timestamp)
        const sha1src = apikey + appkey + roomid + userid + timestamp;
        console.log(sha1src)
        return CryptoJS.SHA1(sha1src).toString(CryptoJS.enc.Hex);
    }

    let localStream = null;
    let localScreenStream = null;
    let remoteStream = null;
    let remoteStreams = {};
    let roomName = '';
    let userTemplate = E('user-template').innerHTML;
    let type = 'rtc';
    let ymrtc = null;

    E('login-type').onchange = function (e) {
        const ele = e.target;
        const options = ele.options;
        const index = ele.selectedIndex;
        type = options[index].value;
    };

    // 自动生成随机用户名
    // let userID = `user-${Math.floor(Math.random() * 1000 + 1)}`;
    // let userID = E('login-user-id').value
    // E('login-user-id').value = userID;

    const handleSucc = () => {
        console.log('init------>succ');
    };
    const handleFail = (err) => {
        console.log('init------>fail', err);
    };

    // 检测浏览器支持
    if (!YMRTC.checkSystemRequirements) {
        alert('您的浏览器不支持RTC，请升级。');
        console.log('您的浏览器不支持RTC，请升级。');
        E('login-form').style.display = 'none';
        return;
    }

    //YMRTC.getDevices 会同时申请摄像头和麦克风权限
    //YMRTC.getAudioDevices 只申请麦克风权限
    YMRTC.getDevices((devices) => {
        const select = E('local-media-devices');
        devices.forEach((d) => {
            const option = document.createElement('option');
            option.innerHTML = d.kind + ':' + d.label;
            option.value = d.deviceId + ':' + d.kind;
            if (d.deviceId === 'default') {
                option.selected = true;
            }
            select.appendChild(option);
        });
    });

    // 登录并加入房间
    E('login').onclick = ()=>{
        login()
    }

    let userId = `user-${Math.floor(Math.random() * 1000 + 1)}`;;
    let roomId = E('login-room-id').value;
    let token = '';
    let timestamp = 0;

    var login =  ()=> {
        console.log('mode:', type);
        userId = E('login-user-id').value;
        roomId = E('login-room-id').value;
        //timestamp = Math.floor(Date.now() / 1000);
        //token = genTokenV3(YM_APPKEY, APIKEY, roomId, userId, timestamp);

        // 初始化
        ymrtc = YMRTC.createClient({
            mode: type,
            codec: 'h264,vp8',
            debug: false
        });

        attachEvents(ymrtc);

        ymrtc.init(
            YM_APPKEY,
            APPID,
            () => {
                console.log('==============>init succ');
            },
            (err) => {
                alert('init fail' + err);
            }
        );
        // roomId = E('login-room-id').value;
        roomName = roomId;
        if (!userId) {
            alert('请输入用户ID。');
            return;
        }
        joinChannel();
    };

    var joinChannel = ()=>{
        // 加入频道
        ymrtc.join(
            token,
            roomId,
            userId,
            timestamp,
            '',
            () => {
                
            },
            () => {
                alert('登录失败。');
            }
        );
    }

    var createLocalStream = (play,publish)=>{
        localStream = ymrtc.createStream({ video: true, audio: true });
        localStream.on('player-status-change', (evt) => {
            console.log('====================>local player-status-change', evt);
            if(evt && !evt.isErrorState && evt.status === "paused"){
                // if(confirm("设备被暂停，是否恢复？")){
                // }
            }
        });
        localStream.init(
            () => {
                E('local-media').style.display = 'inline-block';
                console.log('================>Stream.init succ');
                if(play){
                    console.log("localStream play:"+ localStream._stream.active)
                    localStream.play('local-media', { fit: 'cover', muted: true }, (err) => {
                        if(err && err.toString().indexOf('The operation was aborted') > -1){
                            if(confirm("设备异常断开，是否重新连接？")){
                                createLocalStream(play, publish)
                            }else{

                            }
                        }
                        console.log('==========================>play local', err);
                    });
                }
                if(publish){
                    ymrtc.publish(localStream,()=>{
                        console.log('====================>推流成功');
                    }, (err) => {
                        console.log('====================>推流失败', err);
                    });
                }
            },
            (err) => {
                console.error(err)
                alert('开启设备失败，请检查设备状态: ' + err);
            }
        );
    }

    E('local-media-camera-init').onclick = function (e) {
        createLocalStream()
    };

    E('local-media-screen-init').onclick = function (e) {
        //screenAudio 除了麦克风以外的声音
        //audio 麦克风声音
        localScreenStream = ymrtc.createStream({ screenAudio: true, screen: true, audio: true });
        localScreenStream.on('player-status-change', (evt) => {
            console.log('====================>localScreenStream player-status-change', evt);
        });
        localScreenStream.init(
            () => {
                E('local-screen-media').style.display = 'inline-block';
                console.log('================>localScreenStream Stream.init succ');
            },
            (err) => {
                alert('获取流失败: ' + err);
            }
        );
    };

    E('local-media-mute-audio').onclick = function (e) {
        localStream.muteAudio();
        console.log('======>', localStream.muteAudio());
    };

    E('local-media-unmute-audio').onclick = function (e) {
        localStream.unmuteAudio();
        console.log('======>', localStream.unmuteAudio());
    };

    E('local-media-mute-video').onclick = function (e) {
        localStream.muteVideo();
    };

    E('local-media-unmute-video').onclick = function (e) {
        localStream.unmuteVideo();
    };

    E('local-media-close').onclick = function () {
        localStream.close();
        localStream = null;
    };

    E('local-media-switch-device').onclick = function () {
        const device = E('local-media-devices').value;
        const [deviceId, kind] = device.split(':');
        if (kind.indexOf('output') > -1) {
            return;
        }
        const type = kind.indexOf('video') > -1 ? 'video' : 'audio';
        localStream.switchDevice(
            type,
            deviceId,
            () => {
                console.log('----------------->switchDevice succ');
            },
            (err) => {
                console.log('----------------->switchDevice fail: ', err);
            }
        );
    };

    E('local-media-publish').onclick = function (e) {
        if(localStream){
            ymrtc.publish(localStream,()=>{}, (err) => {
                console.log('====================>localStream 推流失败', err);
            });
        }
        if(localScreenStream){
            ymrtc.publish(localScreenStream,()=>{}, (err) => {
                console.log('====================>localScreenStream 推流失败', err);
            });
        }
    };

    E('local-media-unpublish').onclick = function (e) {
        ymrtc.unpublish(localStream,()=>{}, (err) => {
            console.log('====================>停止推流失败', err);
        });
        if(localScreenStream){
            ymrtc.unpublish(localScreenStream, (err) => {
                console.log('====================>localScreenStream 停止推流失败', err);
            });
        }
    };

    E('local-media-play').onclick = function (e) {
        if(localStream){
        
            var speech = YMRTC.createHark(localStream,{})
            speech.on('volume_change', function (volume,threshold) {
                //音量0.0到1.0
                //  console.error("local volume",volume);
            });

            localStream.play('local-media', { fit: 'cover', muted: true }, (err) => {
                if(err && err.toString().indexOf('The operation was aborted') > -1){
                    //需要重新申请设备
                    if(confirm("设备异常断开，是否重新连接？")){
                        createLocalStream()
                    }else{

                    }
                }
                console.log('==========================>play local', err);
            });
        }
        if(localScreenStream){
            localScreenStream.play('local-screen-media', { fit: 'cover', muted: true }, (err) => {
                if(err && err.toString().indexOf('The operation was aborted') > -1){
                    //需要重新申请设备
                    if(confirm("设备异常断开，是否重新连接？")){
                        //createLocalStream()
                    }else{

                    }
                }
                console.log('==========================>play screen local', err);
            });
        }
    };

    E('local-media-leave').onclick = function () {
        ymrtc.leave(
            () => {
                console.log('========>leave succ');
            },
            (err) => {
                console.log('========>leave fail');
                console.error(err)
            }
        );
    };
    E('local-media-join').onclick = function () {
        joinChannel();
    };

    E('local-media-stop').onclick = function (e) {
        localStream.stop();
    };

    E('video-profile').onchange = function (e) {
        const ele = e.target;
        const options = ele.options;
        const index = ele.selectedIndex;
        const profile = options[index].value;
        localStream.setVideoProfile(profile);
    };

    E('local-set-client-role').onclick = function () {
        const role = E('client-role').value;
        ymrtc.setClientRole(role, (err) => {
            if (err) {
                console.error('=====================>setClientRole', err);
            } else {
                console.log('=====================>setClientRole success');
            }
        });
    };

    E('local-media-subscribe').onclick = function () {
        if (remoteStream) {
            ymrtc.subscribe(remoteStream);
        }
    };

    E('local-media-unsubscribe').onclick = function () {
        if (remoteStream) {
            console.log('=---==============>', remoteStream);
            ymrtc.unsubscribe(remoteStream,()=>{}, (err) => {
                console.log('==================>unsubscribe fail: ', err);
            });
        }
    };

    E('local-media-replacetrack').onclick = function () {
        if (!localStream) {
            return;
        }
        const device = E('local-media-devices').value;
        const [deviceId, kind] = device.split(':');
        if (kind.indexOf('output') > -1) {
            return;
        }
        const type = kind.indexOf('video') > -1 ? 'video' : 'audio';
        const options = {
            video:
                type === 'video'
                    ? {
                          deviceId: deviceId
                      }
                    : false,
            audio:
                type === 'audio'
                    ? {
                          deviceId: deviceId
                      }
                    : false
        };
        navigator.mediaDevices.getUserMedia(options).then((stream) => {
            localStream.replaceTrack(stream.getTracks()[0]);
        });
    };

    E('local-media-removetrack').onclick = function () {
        if (!localStream) {
            return;
        }
        const track = localStream.getVideoTrack();
        if (track) {
            localStream.removeTrack(track);
        }
    };

    E('local-media-addtrack').onclick = function () {
        if (!localStream) {
            return;
        }
        const device = E('local-media-devices').value;
        const [deviceId, kind] = device.split(':');
        if (kind.indexOf('videoinput') <= -1) {
            console.log('================>请选择视频输入源，因为removeTrack接口移除的是视频track');
            return;
        }
        const options = {
            video: { deviceId: deviceId },
            audio: false
        };
        navigator.mediaDevices.getUserMedia(options).then((stream) => {
            localStream.addTrack(stream.getTracks()[0]);
        });
    };

    // 推流地址
    let liveStreamUrl = '';
    E('local-start-live-stream').onclick = function () {
        const baseUrl = E('live-stream-input').value;
        liveStreamUrl = `${baseUrl}/${roomName}`;
        console.log('start-live-stream', liveStreamUrl);
        ymrtc.startLiveStreaming(liveStreamUrl, false);
    };

    E('local-stop-live-stream').onclick = function () {
        if (liveStreamUrl) {
            console.log('stop-live-stream', liveStreamUrl);
            ymrtc.stopLiveStreaming(liveStreamUrl);
        }
    };

    E('sendMesssage').onclick = function () {
        ymrtc.sendMessage("custom message").then(()=>{
            console.log('send custom message success')
        }).catch((e)=>{
            console.error('send custom message error:')
            console.error(e)
        });
    };

    E('kickout').onclick = function (e) {
        let kickUser = E('kicked-user-input').value;
        let kickTime = parseInt(E('kicked-time-input').value);

        ymrtc.kickout(kickUser,
            1,
            kickTime,
            () => {
                console.log('kickout user success')
            },
            () => {
                console.log('kickout user failed')
            });
    };

    function attachEvents(ymrtc) {
        console.log('attachEvents')

        ymrtc.on('media-stats-update',(stream, statsObj) => {
            E('net-stats').innerHTML = "streamid:"+stream.getId()+" " + JSON.stringify(statsObj);
        });

        ymrtc.on('local-media-stats-update',(stream, statsObj) => {
            E('local-net-stats').innerHTML = JSON.stringify(statsObj);
        });

        ymrtc.on('custom-message',(msg) => {
            console.warn('recv custom message:' + msg.msg_content+' from:'+msg.from_user_id)
            alert(msg.msg_content)
        });
        ymrtc.on('stream-subscribed', (evt) => {
            const { stream } = evt;
            console.log('===>stream-subscribed', evt, stream.getId());
            const streamId = stream.getStreamId();
            const elementId = `user-video-${streamId}`;
            stream.play(elementId,{ fit: 'cover', muted: true }, (err) => {
                //如果播放失败，需要改为静音模式
                if (err) {
                    console.warn('========================>stream.play', err);
                }
            });
        });

        ymrtc.on('stream-updated',({ stream }) => {
            // const streamId = stream.getStreamId();
            // const elementId = `user-video-${streamId}`;
            // stream.play(elementId,{ fit: 'cover', muted: true }, (err) => {
            //     //如果播放失败，需要改为静音模式
            // });

            const memberId = stream.getId();
            remoteStreams[memberId] = stream;
        })

        ymrtc.on('stream-removed', (evt) => {
            const { stream } = evt;
            const memberDom = E('user-container-' + stream.getId());
            if (memberDom) {
                E('room').removeChild(memberDom);
            }
        });

        ymrtc.on('peer-leave', ({ uid }) => {
            const memberDom = E('user-container-' + uid);
            if (memberDom) {
                E('room').removeChild(memberDom);
            }
        });

        // 事件监听：用户进入、退出房间
        ymrtc.on('stream-added', function (evt) {
            const { stream } = evt;
            // 有流发送到了房间
            const memberId = stream.getId();
            const streamID = stream.getStreamId();//一个用户可以有多个流
            remoteStreams[memberId] = stream;
            //订阅流，可以结合实际情况考虑
            ymrtc.subscribe(stream,{video:true, audio:true},()=>{}, (err) => {
                console.error('========================>subscribe error', err);
            });
            
            let memberDom = E('user-container-' + memberId);
            if (!memberDom) {
                memberDom = document.createElement('div');
                memberDom.id = 'user-container-' + memberId;
                memberDom.className = 'user-container';
                memberDom.innerHTML = userTemplate.replace(/{{userId}}/g, streamID);
                E('room').appendChild(memberDom);

                // 双击视频全屏
                E('user-video-' + streamID).ondblclick = function (e) {
                    if (e.target.requestFullscreen) e.target.requestFullscreen();
                    else if (e.target.webkitRequestFullScreen) e.target.webkitRequestFullScreen();
                    else if (e.target.mozRequestFullScreen) e.target.mozRequestFullScreen();
                };

                E('user-mute-on-' + streamID).onclick = () => {
                    const stream = remoteStreams[memberId];
                    if (stream) {
                        stream.muteAudio();
                    }else{
                        alert('not found stream')
                    }
                };

                E('user-mute-off-' + streamID).onclick = () => {
                    const stream = remoteStreams[memberId];
                    if (stream) {
                        stream.unmuteAudio();
                        var speech = YMRTC.createHark(stream,{})
                        speech.on('speaking', function () {
                            console.log('Speaking!');
                        });
                        speech.on('volume_change', function (volume,threshold) {
                            //音量0.0到1.0
                            console.log('volume:'+volume);
                        });
                    }else{
                        alert('not found stream')
                    }
                };

                E('user-subscribe-on-' + streamID).onclick = () => {
                    const stream = remoteStreams[memberId];
                    if (stream) {
                        stream.unmuteVideo();
                    }else{
                        alert('not found stream')
                    }
                };

                E('user-subscribe-off-' + streamID).onclick = () => {
                    const stream = remoteStreams[memberId];
                    if (stream) {
                        stream.muteVideo();
                    }else{
                        alert('not found stream')
                    }
                };

                E('user-mute-on-' + streamID).checked = true;
                E('user-subscribe-on-' + streamID).checked = true;
            }
        });

        // 远端用户关闭麦克风
        ymrtc.on('mute-audio', (userId) => {
            console.log('====>mute-audio', userId);
        });
        // 远端用户开启麦克风
        ymrtc.on('unmute-audio', (userId) => {
            console.log('====>unmute-audio', userId);
        });
        // 远端用户关闭扬声器
        ymrtc.on('mute-speaker', (userId) => {
            console.log('====>mute-speaker', userId);
        });
        // 远端用户求情开启您麦克风
        ymrtc.on('request-unmute-audio', (userId) => {
            console.log('====>request-unmute-audio', userId);
        });
        // 远端用户求情关闭您麦克风
        ymrtc.on('request-mute-audio', (userId) => {
            console.log('====>request-mute-audio', userId);
        });
        // 远端用户关闭扬声器
        ymrtc.on('unmute-speaker', (userId) => {
            console.log('====>unmute-speaker', userId);
        });
        // 远端用户开启摄像头
        ymrtc.on('unmute-video', (userId) => {
            console.log('====>unmute-video', userId);
        });
        // 远端用户关闭摄像头
        ymrtc.on('mute-video', (userId) => {
            console.log('====>mute-video', userId);
        });
        // 用户加入退出房间
        ymrtc.on('peer-online', (uid) => {
            console.log('====>peer-online', uid);
        });
        ymrtc.on('member-leave', (uid) => {
            console.log('===>member-leave', uid);
        });
        // 用户退出房间
        ymrtc.on('peer-leave', (stream) => {
            console.log('===>peer-leave', stream.uid);
        });
        // ymrtc.on('stream-subscribed', (evt) => {
        //     const { stream } = evt;
        //     console.log('===>stream-subscribed', evt, stream.getId());
        // });
        ymrtc.on('stream-added', (evt) => {
            const { stream } = evt;
            console.log('================>stream-added', evt, stream.getId());
        });
        ymrtc.on('stream-published', (evt) => {
            const { stream } = evt;
            console.log('====>stream.uid', evt, stream.getId());
        });

        ymrtc.on('stream-unpublished', (evt) => {
            const { stream } = evt;
            console.log('===>stream-unpublished', evt, stream.getId());
        });

        ymrtc.on('client-role-changed', (role) => {
            console.log('===>client-role-changed', role);
        });
        ymrtc.on('liveStreamingFailed', () => {
            console.log('===>liveStreamingFailed');
        });
        ymrtc.on('liveStreamingStarted', () => {
            console.log('===>liveStreamingStarted');
        });

        ymrtc.on('kicked-other-user', (uid) => {
            // const {userId} = uid
            alert('other be kicked ' + uid);
        });
        ymrtc.on('kicked-user', (uid) => {
            alert('you be kicked, ' + uid);
        });

        ymrtc.on('kickout-failed', (msg) => {
            const { uid, err_code, err_msg } = msg
            console.log(msg)
            alert(uid+ ' kickout fail err[' +err_code+ '], ' + err_msg);
        });

        ymrtc.on('joinroom-failed', (msg) => {
            const { uid, err_code, err_msg } = msg
            console.log(msg)
            alert(uid+ ' joinroom fail, err[' +err_code+ '], ' + err_msg);
        });

        ymrtc.on('joined', (msg) => {
            console.log(msg)
            // alert(uid+ ' joinroom fail, err[' +err_code+ '], ' + err_msg);
            E('not-log').style.display = 'none';
            E('logged').style.display = 'inline';
            E('login-form').style.display = 'none';
            E('rooms-container').style.display = 'block';
            E('user-id').innerHTML = userId;
            E('local-media-indicator').style.display = 'block';
            E('signaling-status').innerHTML = '已连接';
        });

        // 用于测试，监听所有事件
        ymrtc.on('*', (eventName, payload) => {
            console.log('ymrtc event:', eventName, payload);
            if(eventName === 'error' && payload.reason === 'SOCKET_DISCONNECTED')
            {
                login()
            }

            // if(eventName === 'reconnected')
            // {
            //     //有可能从休眠状态返回，直接尝试重开设备
            //     var play = true;
            //     var publish = true;
            //     //必须用户点击触发才能开启摄像头
            //     if(!ymrtc.isLive()){
            //         if(confirm("网络重联成功，是否重新开启设备？")){
            //             ymrtc.unpublish(localStream,()=>{}, (err) => {
            //                 console.log('====================>停止推流失败', err);
            //             });
            //             createLocalStream(play, publish)
            //         }
            //     }
            // }
        });
    }
})();
